@@ -2,7 +2,9 @@
 from . excel_levity_cls import excel_levity_cls
 from . flux_cls import flux_cls
 from . flux_row_cls import flux_row_cls
+from . log_cls import log_cls
 
 __all__ = ['excel_levity_cls',
            'flux_cls',
-           'flux_row_cls']
+           'flux_row_cls',
+           'log_cls']
