@@ -12,59 +12,165 @@ from .. excel_com.excel_address import col_number
 from .. excel_com.excel_address import max_cols as excel_max_cols
 from .. excel_com.excel_address import max_rows as excel_max_rows
 
-from .workbook import app_to_foreground
-from .excel_constants import *
+from . workbook import app_to_foreground
+from . excel_constants import *
 
 
-def get_worksheet(wb, tab_name,
+def get_worksheet(wb,
+                  ws,
                   *,
                   clear_filter=False,
                   activate=False):
 
-    if tab_name.__class__.__name__ in {'CDispatch', '_Worksheet'}:
-        return tab_name
+    if ws.__class__.__name__ in {'CDispatch', '_Worksheet'}:
+        return ws
 
     if wb is None:
-        raise AssertionError("Cannot retrieve tab: '{}' Excel workbook has not been initialized!".format(tab_name))
+        raise AssertionError("Excel workbook has not been set, cannot retrieve worksheet: '{}'".format(ws))
 
-    ws = None
-    if isinstance(tab_name, str):
-        for ws in wb.Sheets:
-            if ws.Name.lower() == tab_name.lower():
-                break
-
-    elif isinstance(tab_name, int):
-        ws = wb.sheets[tab_name]
-
-    if ws is None:
-        raise NameError("tab: '{}' not found in workbook: '{}'".format(tab_name, wb.name))
-
-    ws.Visible = True
+    try:
+        ws = wb.Sheets[ws]
+    except com_error as e:
+        raise NameError("'{}' worksheet not found in '{}'".format(ws, wb.Name)) from e
 
     if clear_filter:
         clear_worksheet_filter(ws)
 
     if activate:
+        ws.Visible = True
         app_to_foreground(ws.Application)
         ws.Activate()
 
     return ws
 
 
+def first_row(excel_range, default_r=1):
+    search = range_find(excel_range,
+                        what='*',
+                        look_at=xlPart,
+                        search_order=xlByRows,
+                        search_direction=xlNext)
+
+    if search:
+        r = max(search.Row, default_r)
+    else:
+        r = default_r
+
+    return r
+
+
+def last_row(excel_range, default_r=1):
+    search = range_find(excel_range,
+                        what='*',
+                        look_at=xlPart,
+                        search_order=xlByRows,
+                        search_direction=xlPrevious)
+
+    if search:
+        r = max(search.Row, default_r)
+    else:
+        r = default_r
+
+    return r
+
+
+def first_col(excel_range, default_c='A'):
+
+    search = range_find(excel_range,
+                        what='*',
+                        look_at=xlPart,
+                        search_order=xlByColumns,
+                        search_direction=xlNext)
+
+    if search:
+        c = max(search.Column, col_number(default_c))
+        c_lttr = col_letter(c)
+    else:
+        c_lttr = default_c
+
+    return c_lttr
+
+
+def last_col(excel_range, default_c='A'):
+
+    search = range_find(excel_range,
+                        what='*',
+                        look_at=xlPart,
+                        search_order=xlByColumns,
+                        search_direction=xlPrevious)
+
+    if search:
+        c = max(search.Column, col_number(default_c))
+        c_lttr = col_letter(c)
+    else:
+        c_lttr = default_c
+
+    return c_lttr
+
+
+def range_find(excel_range,
+               what,
+               after=None,
+               look_in=xlValues,
+               look_at=xlWhole,
+               search_order=xlByRows,
+               search_direction=xlPrevious,
+               match_case=False):
+
+    if after is None:
+        if search_direction == xlNext:
+            after = last_cell(excel_range)
+        elif search_direction == xlPrevious:
+            after = first_cell(excel_range)
+
+    search = excel_range.Find(what,
+                              after,
+                              look_in,
+                              look_at,
+                              search_order,
+                              search_direction,
+                              match_case)
+
+    return search
+
+
+def first_cell(excel_range):
+    return excel_range.Cells(1)
+
+
+def last_cell(excel_range):
+    """
+    excel_range.Cells(excel_range.Cells.Count) is not always reliable
+    and can cause an overflow error, but when excel_range.Address
+    only provides row address, something like '$1:$20', this method must be used
+
+    # '$1:$20' = excel_range.Address
+    # excel_range.Parent.Rows(a)
+    """
+    try:
+        a = excel_range.Address.split(':')[-1]
+        return excel_range.Parent.Range(a)
+    except com_error:
+        pass
+
+    return excel_range.Cells(excel_range.Cells.Count)
+
+
 def is_filtered(ws):
-    if ws.AutoFilter is not None:
-        return ws.AutoFilter.FilterMode
+    return (ws.AutoFilter is not None) and bool(ws.AutoFilter.FilterMode)
 
 
+# noinspection PyProtectedMember
 def clear_worksheet_filter(ws):
     if is_filtered(ws):
-        # noinspection PyProtectedMember
         ws._AutoFilter.ShowAllData()
 
 
 def is_range_empty(excel_range):
-    num_cells = int(excel_range.Application.WorksheetFunction.CountA(excel_range))
-    return num_cells == 0
+    num_blank = excel_range.Application.WorksheetFunction.CountBlank(excel_range)
+    num_cells = excel_range.Cells.Count
+
+    return num_blank == num_cells
 
 
 def activate_sheet(ws):
@@ -97,7 +203,7 @@ def __convert_excel_errors(excel_range):
     m = [list(row) for row in m]
 
     try:
-        range_errors = excel_range.SpecialCells(xl_cell_type_formulas, xl_errors)
+        range_errors = excel_range.SpecialCells(xlCellTypeFormulas, xlErrors)
         cell = first_cell(excel_range)
         r_0  = cell.Row
         c_0  = cell.Column
@@ -114,143 +220,39 @@ def __convert_excel_errors(excel_range):
 
 
 def __excel_friendly_matrix(m):
-    """ modify matrix values so they can be written to an excel range without error """
+    """ modify matrix values so they can be written to an Excel range without error
+    TODO:
+        profile / optimize function
+        convert date to datetime?
+    """
     num_cols = len(m[0])
+    num_rows = len(m)
 
     if num_cols > excel_max_cols:
         raise ValueError("number of columns ({:,}) exceeds Excel's column limit\n"
                          "(did you mean to transpose this matrix?)".format(num_cols))
 
-    if len(m) > excel_max_rows:
-        raise ValueError("number of rows ({:,}) exceeds Excel's row limit".format(len(m)))
+    if num_rows > excel_max_rows:
+        raise ValueError("number of rows ({:,}) exceeds Excel's row limit".format(num_rows))
 
     for r, row in enumerate(m):
         if len(row) != num_cols:
-            raise ValueError('jagged column (at row {:,}), cannot write to Excel'.format(r))
+            _m_ = '\n\t'.join([repr(_row_) for _row_ in m[r - 1:r + 2]])
+            raise ValueError('cannot write to Excel, jagged column in matrix'
+                             '\nrow {:,}\n\n\t{}'.format(r, _m_))
 
-        row_ = []
+        _row_ = []
         for c, v in enumerate(row):
-            if isinstance(v, (bool, int, float, str)) or v is None:
+            if (v is None) or isinstance(v, (bool, int, float, str)):
                 pass
             elif isinstance(v, date):
                 v = datetime(v.year, v.month, v.day)
             else:
                 v = repr(v)
 
-            row_.append(v)
+            _row_.append(v)
 
-        yield row_
-
-
-def range_find(excel_range,
-               what,
-               after=None,
-               look_in=xl_values,
-               look_at=xl_whole,
-               search_order=xl_by_rows,
-               search_direction=xl_previous,
-               match_case=False):
-
-    if after is None:
-        if search_direction == xl_next:
-            after = last_cell(excel_range)
-        elif search_direction == xl_previous:
-            after = first_cell(excel_range)
-
-    search = excel_range.Find(what,
-                              after,
-                              look_in,
-                              look_at,
-                              search_order,
-                              search_direction,
-                              match_case)
-
-    return search
-
-
-def first_cell(excel_range):
-    return excel_range.Cells(1)
-
-
-def last_cell(excel_range):
-    """ excel_range.Cells(excel_range.Cells.Count) causes an overflow error
-    this is not always reliable when excel_range.Cells() requires both row and column arguments
-    """
-    a = excel_range.Address.split(':')[-1]
-    return excel_range.Parent.Range(a)
-
-
-def first_row(excel_range, default_r=1):
-    search = range_find(excel_range,
-                        what='*',
-                        search_order=xl_by_rows,
-                        search_direction=xl_next)
-
-    if search:
-        r = max(search.Row, default_r)
-    else:
-        r = default_r
-
-    return r
-
-
-def last_row(excel_range, default_r=1):
-    search = range_find(excel_range,
-                        what='*',
-                        search_order=xl_by_rows,
-                        search_direction=xl_previous)
-
-    if search:
-        r = max(search.Row, default_r)
-    else:
-        r = default_r
-
-    return r
-
-
-def first_col(excel_range, default_c='A'):
-    search = range_find(excel_range,
-                        what='*',
-                        search_order=xl_by_columns,
-                        search_direction=xl_next)
-
-    if search:
-        c = max(search.Column, col_number(default_c))
-        c_lttr = col_letter(c)
-    else:
-        c_lttr = default_c
-
-    return c_lttr
-
-
-def last_col(excel_range, default_c='A'):
-    search = range_find(excel_range,
-                        what='*',
-                        search_order=xl_by_columns,
-                        search_direction=xl_previous)
-
-    if search:
-        c = max(search.Column, col_number(default_c))
-        c_lttr = col_letter(c)
-    else:
-        c_lttr = default_c
-
-    return c_lttr
-
-
-def first_non_blank_col(excel_range, default_c='A'):
-    search = range_find(excel_range,
-                        what='',
-                        search_order=xl_by_columns,
-                        search_direction=xl_next)
-
-    if search:
-        c = max(search.Column - 1, col_number(default_c))
-        c_lttr = col_letter(c)
-    else:
-        c_lttr = default_c
-
-    return c_lttr
+        yield _row_
 
 
 def parse_range(excel_range):
