@@ -1,89 +1,94 @@
 
-import os
+import _pickle as cpickle
+
 import csv
 import json
-import pickle
+import os
 import shutil
 
-from glob import glob
 from datetime import datetime
+from glob import glob
 
-from . iter import is_iterable
-from . iter import generator_to_list
-from . iter import assert_iteration_depth
-
-from . text import repr_
-from . text import p_json_dumps
+from .text import p_json_dumps
+from .text import repr_
 
 
-def read_file(path, encoding=None):
+def read_file(path, encoding=None, mode='r'):
     """
-    supports
+    assumes text file unless extension is in these specialized formats:
         .csv
         .json
         .flux
-        (text-file encodings)
+        .pkl
+        .pickle
     """
     assert_path_exists(path)
+    extn = file_extension(path, include_dot=True)
+
+    if extn.startswith('.xls') or extn in {'.7z', '.gzip', '.hd5'}:
+        raise NotImplementedError
 
-    extn = file_extension(path)
     if extn == '.csv':
-        with open(path, 'r', encoding=encoding) as f:
+        with open(path, mode, encoding=encoding) as f:
             return list(csv.reader(f))
 
     if extn == '.json':
-        with open(path, 'r', encoding=encoding) as f:
+        with open(path, mode, encoding=encoding) as f:
             return json.load(f)
 
-    if extn == '.flux':
-        with open(path, 'rb') as f:
-            return pickle.load(f)
+    if extn in {'.flux', '.pkl', '.pickle'}:
+        if not mode.endswith('b'):
+            mode += 'b'
+        with open(path, mode) as f:
+            return cpickle.load(f)
 
-    with open(path, 'r', encoding=encoding) as f:
+    with open(path, mode, encoding=encoding) as f:
         return f.read()
 
 
-def write_file(path, data, mode='w', encoding=None):
+def write_file(path, data, encoding=None, mode='w'):
     """
-    supports
+    assumes text file unless extension is in these specialized formats:
         .csv
         .json
         .flux
-        (text-file encodings)
+        .pkl
+        .pickle
     """
-    extn = file_extension(path)
+    extn = file_extension(path, include_dot=True)
+
+    if extn.startswith('.xls') or extn in {'.7z', '.gzip', '.hd5'}:
+        raise NotImplementedError
 
     if extn == '.csv':
-        data = generator_to_list(data, recurse=True)
-        assert_iteration_depth(data, 2)
-
         with open(path, mode, encoding=encoding) as f:
             csv.writer(f, lineterminator='\n').writerows(data)
+        return
 
-    elif extn == '.json':
+    if extn == '.json':
         if not isinstance(data, str):
-            data = p_json_dumps(data, ensure_ascii=False)
+            data = p_json_dumps(data, ensure_ascii=(encoding is None))
 
         with open(path, mode, encoding=encoding) as f:
             f.write(data)
 
-    elif extn == '.flux':
-        with open(path, mode + 'b') as f:
-            pickle.dump(data, f)
+        return
 
-    else:
-        with open(path, mode, encoding=encoding) as f:
-            if is_iterable(data):
-                data = repr_(data, quotes=True, concat=',\n')
-                # data = repr(data)
+    if extn in {'.flux', '.pkl', '.pickle'}:
+        if not mode.endswith('b'):
+            mode += 'b'
 
-            f.write(data)
+        with open(path, mode) as f:
+            cpickle.dump(data, f)
 
+        return
 
-def make_dirs(f_dir):
-    if not os.path.isdir(f_dir):
-        if not os.path.exists(f_dir):
-            os.makedirs(f_dir)
+    if not isinstance(data, str):
+        # convert data to string: f.write() is much faster than f.writelines()
+        data = repr_(data, concat='\n', quotes=False, wrap=False)
+
+    with open(path, mode, encoding=encoding) as f:
+        f.write(data)
 
 
 def clear_dir(f_dir, allow_not_exist=False):
@@ -107,6 +112,12 @@ def clear_dir(f_dir, allow_not_exist=False):
 def copy_dir(s_dir, d_dir, exclude_dirs=None):
     exclude_dirs = exclude_dirs or set()
 
+    s_dir = standardize_dir(s_dir)
+    d_dir = standardize_dir(d_dir)
+
+    if not os.path.exists(d_dir):
+        os.makedirs(d_dir)
+
     for file_content in os.listdir(s_dir):
         if file_content in exclude_dirs:
             continue
@@ -130,34 +141,28 @@ def file_last_modified(path):
     return datetime.fromtimestamp(unix_t)
 
 
-def is_modified_today(path):
-    d = file_last_modified(path).date()
-    t = datetime.today().date()
-
-    return d == t
-
-
-def standardize_dir(f_dir):
+def standardize_dir(f_dir, pathsep='/'):
+    """ lowercase and make sure directory follows predictable pattern
+    pathsep=os.path.sep?
     """
-    replace all '\' with '/'
-    make sure dir ends with '/'
-    """
-    f_dir = (f_dir.strip(' \n')
-                  .replace('\\', '/')
-                  .lower())
+    f_dir = (f_dir.replace('\\', pathsep)
+                  .replace('/', pathsep)
+                  .lower()
+                  .strip())
 
-    if not f_dir.endswith('/'):
-        f_dir += '/'
+    if not f_dir.endswith(pathsep):
+        f_dir += pathsep
 
     return f_dir
 
 
 def standardize_file_name(f_name):
-    return f_name.strip(' \n').lower()
+    return f_name.lower().strip()
 
 
 def standardize_path(path):
     f_dir, f_name = parse_path(path)
+
     f_dir  = standardize_dir(f_dir)
     f_name = standardize_file_name(f_name)
 
@@ -165,9 +170,12 @@ def standardize_path(path):
 
 
 def sanatize_file_name(f_name):
-    if not f_name:
-        raise IOError("file name is blank: '{}'".format(f_name))
+    """
+    replace illegal Windows file name characters with '-'
 
+    (there's an additional set of path characters that are
+     illegal for Windows' built-in compression)
+    """
     invalid_chrs = {'\\': '-',
                     '/':  '-',
                     ':':  '-',
@@ -176,12 +184,15 @@ def sanatize_file_name(f_name):
                     '<':  '-',
                     '>':  '-',
                     '|':  '-',
-                    '"':  '-',
-                    "'":  '-'}
+                    '"':  '-'}
 
     for k, v in invalid_chrs.items():
         f_name = f_name.replace(k, v)
 
+    f_name = f_name.strip()
+    if not f_name:
+        raise IOError('file name is blank')
+
     return f_name
 
 
@@ -207,6 +218,7 @@ def assert_path_exists(path):
 
 
 def parse_path(path):
+
     if os.path.isdir(path):
         f_dir  = path
         f_name = ''
@@ -221,19 +233,18 @@ def file_extension(f_name, include_dot=True):
     if include_dot is False:
         extn = extn.replace('.', '')
 
+    extn = extn.lower().strip()
+
     return extn
 
 
 def apply_file_extension(f_name, extn):
-    if f_name.endswith(extn):
-        return f_name
-
     if not extn.startswith('.'):
         extn = '.' + extn
 
-    if not f_name.endswith(extn):
-        f_name = f_name + extn
+    if f_name.endswith(extn):
+        return f_name
 
-    return f_name
+    return f_name + extn
 
 
