@@ -1,39 +1,77 @@
-
+import re
 import json
 
 from timeit import default_timer
 from datetime import date
 from collections import Iterable
 
-nbsp = chr(160)
-nonstandard_chars = "\"\'\n\t" + nbsp
-
 
 def print_runtime(f):
-
     def runtime_wrapper(*args, **kwargs):
-        f_name = f.__module__.split('.')[-1]
-        f_name = '{}.{}'.format(f_name, f.__name__)
-
         tic = default_timer() * 1000
-
-        if kwargs:
-            result = f(*args, **kwargs)
-        elif args:
-            result = f(*args)
-        else:
-            result = f()
-
+        result = f(*args, **kwargs)
         toc = default_timer() * 1000
-        elapsed = -(tic - toc)
 
-        print_unicode('   τ: @{}: {}'.format(f_name, format_ms(elapsed)))
+        elapsed = -(tic - toc)
+        print_unicode('\tτ: @{}: {}'.format(function_name(f), format_ms(elapsed)))
 
         return result
 
     return runtime_wrapper
 
 
+def print_performance(f=None, *, repeat=3):
+    """
+    stolen and modified from
+    https://github.com/realpython/materials/blob/master/pandas-fast-flexible-intuitive/tutorial/timer.py
+    """
+    import gc
+    import functools
+    import itertools
+
+    functools_repeat = functools.partial(itertools.repeat, None)
+
+    def performance_wrapper(_f_):
+
+        @functools.wraps(_f_)
+        def functools_wrapper(*args, **kwargs):
+            gcold = gc.isenabled()
+            gc.disable()
+
+            result = None
+            best   = None
+            total  = 0
+
+            try:
+                for _ in functools_repeat(repeat):
+                    tic = default_timer() * 1000
+                    result = _f_(*args, **kwargs)
+                    toc = default_timer() * 1000
+
+                    elapsed = -(tic - toc)
+                    if best is None:
+                        best = elapsed
+                    else:
+                        best = min(best, elapsed)
+
+                    total += elapsed
+
+                average = total / repeat
+                print_unicode('\tτ: @{}\n\t\taverage:   {}\n\t\tbest:      {}\n'
+                              .format(function_name(_f_), format_ms(average), format_ms(best)))
+            finally:
+                if gcold:
+                    gc.enable()
+
+            return result
+        return functools_wrapper
+
+    if f is None:
+        return performance_wrapper
+    else:
+        return performance_wrapper(f)
+
+
 def format_ms(ms):
     m, s = divmod(ms / 1000, 60)
     h, m = divmod(m, 60)
@@ -54,78 +92,8 @@ def format_ms(ms):
     return f_ms
 
 
-def print_timeit(f, *, repeat=3, number=1000):
-    """
-    stolen and modified from
-    https://github.com/realpython/materials/blob/master/pandas-fast-flexible-intuitive/tutorial/timer.py
-
-    Decorator: prints time from best of `repeat` trials.
-
-    Mimics `timeit.repeat()`, but avg. time is printed.
-    Returns function result and prints time.
-
-    You can decorate with or without parentheses, as in
-    Python's @dataclass class decorator.
-
-    kwargs are passed to `print()`.
-    """
-    import gc
-    import functools
-    import itertools
-
-    _repeat = functools.partial(itertools.repeat, None)
-
-    def wrap(func):
-
-        @functools.wraps(func)
-        def performance_wrapper(*args, **kwargs):
-            # Temporarily turn off garbage collection during the timing.
-            # Makes independent timings more comparable.
-            # If it was originally enabled, switch it back on afterwards.
-            gcold = gc.isenabled()
-            gc.disable()
-
-            result = None
-
-            try:
-                # Outer loop - the number of repeats.
-                trials = []
-                for _ in _repeat(repeat):
-                    # Inner loop - the number of calls within each repeat.
-                    total = 0
-                    for _ in _repeat(number):
-                        start = default_timer()
-                        result = func(*args, **kwargs)
-                        end = default_timer()
-                        total += end - start
-
-                    trials.append(total)
-
-                # We want the *average time* from the *best* trial.
-                # For more on this methodology, see the docs for
-                # Python's `timeit` module.
-                #
-                # "In a typical case, the lowest value gives a lower bound
-                # for how fast your machine can run the given code snippet;
-                # higher values in the result vector are typically not
-                # caused by variability in Python’s speed, but by other
-                # processes interfering with your timing accuracy."
-                best = min(trials) / number
-
-                print("Best of {} trials with {} function calls per trial:"
-                      .format(repeat, number))
-                print("Function `{}` ran in average of {:0.3f} seconds."
-                      .format(func.__name__, best), end="\n\n")
-            finally:
-                if gcold:
-                    gc.enable()
-
-            return result
-
-        return performance_wrapper
-
-    # Syntax trick from Python @dataclass
-    return wrap(f)
+def function_name(f):
+    return '{}.{}'.format(f.__module__.split('.')[-1], f.__name__)
 
 
 def repr_(sequence, concat=', ', quotes=False, wrap=True):
@@ -182,28 +150,32 @@ def repr_(sequence, concat=', ', quotes=False, wrap=True):
 
 
 def sanitize(v):
+    """
+    convert value to string and remove common characters that can make
+    string more difficult to work with (quotes and whitespace)
+
+    (the html non-breaking space (&nbsp) is particularly annoying, but dealt
+    with by the regex whitespace character class)
+    """
+    non_standard_re = re.compile(r'[\"\'\s]+')
+
     s = value_to_string(v)
-    s = remove_multiple(s, nonstandard_chars)
-    s = s.strip()
+    s = non_standard_re.sub('', s).strip()
 
     return s
 
 
 def value_to_string(v):
-    if isinstance(v, str):
-        return v
-
     if v is None:
-        v = ''
-    elif isinstance(v, date):
-        v = v.strftime('%Y-%m-%d')
-    else:
-        v = str(v)
+        return ''
 
-    return v
+    return str(v)
 
 
 def remove_multiple(s, substrs, first_only=False):
+    if s == '':
+        return s
+
     for sub in substrs:
         if first_only:
             s = s.replace(sub, '', 1)
@@ -233,6 +205,7 @@ def p_json_dumps(o, indent=4, ensure_ascii=False):
         dates
         sets
     """
+
     def unhandled_conversion(_o):
         if isinstance(_o, date):
             return _o.isoformat()
@@ -242,7 +215,7 @@ def p_json_dumps(o, indent=4, ensure_ascii=False):
 
         raise TypeError('cannot convert type to json ' + repr(_o))
 
-    s = json.dumps(o, indent=indent,  default=unhandled_conversion, ensure_ascii=ensure_ascii)
+    s = json.dumps(o, indent=indent, default=unhandled_conversion, ensure_ascii=ensure_ascii)
 
     return s
 
@@ -259,7 +232,7 @@ def to_ascii(s):
         s = s.replace(old, new)
 
     s = (s.encode('ascii', errors='backslashreplace')
-          .decode('ascii'))
+         .decode('ascii'))
 
     return s
 
@@ -272,6 +245,4 @@ def print_unicode(s):
 
 
 def vengeance_message(s):
-    print_unicode('   ν: {}'.format(s))
-
-
+    print_unicode('\tν: {}'.format(s))
